MODULE OberonConsoleUtils;

(** AUTHOR "Yaroslav Romanchenko (SAGE) http://sage.com.ua/ rapturize@gmail.com";
	PURPOSE "Different utilities usefull for console"; *)

IMPORT
	Commands, Options, Files, Strings, Streams, Codecs, Texts;
	
CONST
	CRLF = ''0DX''0AX'';
	
TYPE
	String = Strings.String;
	Chars = ARRAY OF CHAR;
	
OPERATOR "+"(CONST a1, a2: Chars): String;
VAR str: String;
BEGIN
	NEW(str, Strings.Length(a1) + Strings.Length(a2) + 1);
	Strings.Concat(a1, a2, str^);
	RETURN str
END "+";

OPERATOR "+"(i: SIGNED64; CONST a2: Chars): String;
VAR
	a1: ARRAY 32 OF CHAR;
	str: String;
BEGIN
	Strings.IntToStr(i, a1);
	NEW(str, Strings.Length(a1) + Strings.Length(a2) + 1);
	Strings.Concat(a1, a2, str^);
	RETURN str
END "+";

OPERATOR "+"(CONST a1: Chars; i: SIGNED64): String;
VAR
	a2: ARRAY 32 OF CHAR;
	str: String;
BEGIN
	Strings.IntToStr(i, a2);
	NEW(str, Strings.Length(a1) + Strings.Length(a2) + 1);
	Strings.Concat(a1, a2, str^);
	RETURN str
END "+";

VAR
	bSilent: BOOLEAN;

PROCEDURE Message(log: Streams.Writer; strMessage: String);
BEGIN
	IF ~bSilent THEN
		log.String(strMessage^)
	END
END Message;

PROCEDURE ReadFile(log: Streams.Writer; CONST aFileName: ARRAY OF CHAR; VAR iLength: SIZE): String;
VAR
	file: Files.File;
	reader: Files.Reader;
	str: String;
BEGIN
	str := NIL;
	file := Files.Old(aFileName);
	IF file = NIL THEN
		Message(log, 'Bad file name: "' + aFileName + '"' + CRLF)
	ELSE
		NEW(str, file.Length() + 1);
		Files.OpenReader(reader, file, 0);
		reader.Bytes(str^, 0, LEN(str^), iLength)
	END;
	RETURN str
FINALLY
	IF file # NIL THEN
		file.Close()
	END
END ReadFile;

PROCEDURE WriteFile(log: Streams.Writer; CONST aFileName, aContent: ARRAY OF CHAR; iPos, iLength: SIZE);
VAR
	file: Files.File;
	writer: Files.Writer;
BEGIN
	Message(log, 'Writing file "' + aFileName + '" ...' + CRLF);
	file := Files.New(aFileName);
	IF file = NIL THEN
		Message(log, 'Unable to create file: "' + aFileName + '"' + CRLF)
	ELSE
		Files.OpenWriter(writer, file, 0);
		writer.Bytes(aContent, iPos, iLength)
	END
FINALLY
	IF file # NIL THEN
		writer.Update();
		Files.Register(file);
		file.Close()
	END
END WriteFile;

PROCEDURE DecodeFile(log: Streams.Writer; CONST aFileName: ARRAY OF CHAR; decoder: Codecs.TextDecoder; res: INTEGER): Texts.Text;
VAR
	file: Files.File;
	reader: Files.Reader;
	text: Texts.Text;
BEGIN
	res := Codecs.ResFailed;
	text := NIL;
	file := Files.Old(aFileName);
	IF file = NIL THEN
		Message(log, 'Bad file name: "' + aFileName + '"' + CRLF);
		Message(log, aFileName + CRLF);
		Message(log, aFileName + CRLF)
	ELSE
		Files.OpenReader(reader, file, 0);
		decoder.Open(reader, res);
		IF res = Codecs.ResOk THEN
			text := decoder.GetText()
		END
	END;
	RETURN text;
FINALLY
	IF file # NIL THEN
		file.Close()
	END
END DecodeFile;

PROCEDURE EncodeFile(log: Streams.Writer; CONST aFileName: ARRAY OF CHAR; encoder: Codecs.TextEncoder; text: Texts.Text; bBOM: BOOLEAN; res: INTEGER);
VAR
	file: Files.File;
	writer: Files.Writer;
BEGIN
	res := Codecs.ResFailed;
	Message(log, 'Writing file "' + aFileName + '" ...' + CRLF);
	file := Files.New(aFileName);
	IF file = NIL THEN
		Message(log, 'Unable to create file: "' + aFileName + '"' + CRLF)
	ELSE
		Files.OpenWriter(writer, file, 0);
		
		(* BOM *)
		IF bBOM THEN
			writer.Char(0EFX);
			writer.Char(0BBX);
			writer.Char(0BFX);
			writer.Update();
		END;
		
		encoder.Open(writer);
		encoder.WriteText(text, res);
	END
FINALLY
	IF file # NIL THEN
		writer.Update();
		Files.Register(file);
		file.Close()
	END
END EncodeFile;

PROCEDURE StringReplace(log: Streams.Writer; CONST aString, aOldPat, aNewPat: ARRAY OF CHAR; bIgnoreCase: BOOLEAN; VAR nReplaced: SIZE): Strings.Buffer;
VAR
	length: SIZE;
	from, pos, posLast: SIZE;
	buffer: Strings.Buffer;
	writer: Streams.Writer;
BEGIN
	length := LEN(aString);
	NEW(buffer, length + length DIV 16);
	writer := buffer.GetWriter();
	from := 0;
	posLast := -1;
	nReplaced := 0;
	REPEAT
		pos := Strings.GenericPos(aOldPat, from, aString, bIgnoreCase, FALSE);
		IF pos >= 0 THEN
			posLast := pos;
			INC(nReplaced);
			Message(log, 'pos: ' + pos + CRLF);
			
			IF pos > 0 THEN
				writer.Bytes(aString, from, pos - from)
			END;
			writer.Bytes(aNewPat, 0, Strings.Length(aNewPat));
			from := pos + Strings.Length(aOldPat)
		ELSIF posLast >= 0 THEN (* Write tail *)
			writer.Bytes(aString, from, length - from)
		END
	UNTIL pos < 0;
	IF nReplaced = 0 THEN (* Nothing replaced, just copy *)
		writer.Bytes(aString, 0, length)
	END;
	RETURN buffer
END StringReplace;

(** Usage: ConsoleUtils.ReplaceStringInFile [--silent] [--case] --old="old string" --new="new string" <Input file> [<Ouput file>] *)
PROCEDURE ReplaceStringInFile*(context: Commands.Context);
VAR
	bOptionsOk, bIgnoreCase: BOOLEAN;
	options: Options.Options;
	aOldPat, aNewPat: ARRAY 256 OF CHAR;
	aFileNameIn, aFileNameOut: Files.FileName;
	strContent: String;
	buffer: Strings.Buffer;
	nRead, nReplaced: SIZE;
BEGIN
	NEW(options);
	options.Add("o", "old", Options.String);
	options.Add("n", "new", Options.String);
	options.Add("s", "silent", Options.Flag);
	options.Add("c", "case", Options.Flag); (* case sensitive *)
	bOptionsOk := options.Parse(context.arg, context.out) &
		options.GetString("old", aOldPat) & options.GetString("new", aNewPat);
	IF bOptionsOk THEN
		context.arg.SkipWhitespace();
		context.arg.String(aFileNameIn);
		bOptionsOk := aFileNameIn # ""
	END;
	
	bSilent := options.GetFlag("silent");
	bIgnoreCase := ~options.GetFlag("case");

	IF ~bOptionsOk THEN
		Message(context.out, CRLF + "Required parameters not provided. Usage:" + CRLF +
			'ConsoleUtils.ReplaceStringInFile --old="old string" --new="new string" <Input file> [<Ouput file>]' + CRLF)
	ELSE
		context.arg.SkipWhitespace();
		context.arg.String(aFileNameOut);
		strContent := ReadFile(context.out, aFileNameIn, nRead);
		IF strContent # NIL THEN
			Message(context.out, 'File "' + aFileNameIn + '" opened, ' + nRead + ' bytes read.' + CRLF);
			buffer := StringReplace(context.out, strContent^, aOldPat, aNewPat, bIgnoreCase, nReplaced);
			Message(context.out, nReplaced + ' occurences of pattern "' + aOldPat + '" found.' + CRLF);
			IF aFileNameOut # "" THEN
				WriteFile(context.out, aFileNameOut, buffer.GetString()^, 0, buffer.GetLength() - 1)
			ELSE
				WriteFile(context.out, aFileNameIn, buffer.GetString()^, 0, buffer.GetLength() - 1)
			END
		END
	END
END ReplaceStringInFile;

(** Usage: ConsoleUtils.ChangeFileEncoding [--silent] --old="old encoding" --new="new encoding" <Input file> [<Ouput file>]

	For list of supported encodings see Configuration.XML
	For moment of writing this utility the list was:
		Oberon, BBT, UTF-8, HEX, ISO8859-1, ISO-8859-5, KOI8-R, KOI8-U", CP1251, CP866, CP1250
		
	For proper work of utility the required encodings MUST be registered in Configuration.XML!
		
 *)
PROCEDURE ChangeFileEncoding*(context: Commands.Context);
VAR
	bOptionsOk: BOOLEAN;
	bBOM: BOOLEAN;
	options: Options.Options;
	aOldEncoding, aNewEncoding: ARRAY 256 OF CHAR;
	aFileNameIn, aFileNameOut: Files.FileName;
	textDecoder: Codecs.TextDecoder;
	textEncoder: Codecs.TextEncoder;
	text: Texts.Text;
	res: INTEGER;
BEGIN
	NEW(options);
	options.Add("o", "old", Options.String);
	options.Add("n", "new", Options.String);
	options.Add("s", "silent", Options.Flag);
	options.Add("b", "bom", Options.Flag);
	bOptionsOk := options.Parse(context.arg, context.out) &
		options.GetString("old", aOldEncoding) & options.GetString("new", aNewEncoding);
	IF bOptionsOk THEN
		context.arg.SkipWhitespace();
		context.arg.String(aFileNameIn);
		bOptionsOk := aFileNameIn # ""
	END;
	
	bSilent := options.GetFlag("silent");
	
	bBOM := options.GetFlag("bom") & (aNewEncoding = 'UTF-8');

	IF ~bOptionsOk THEN
		Message(context.out, CRLF + "Required parameters not provided. Usage:" + CRLF +
			'ConsoleUtils.ChangeFileEncoding --old="old encoding" --new="new encoding" <Input file> [<Ouput file>]' + CRLF)
	ELSE
	
		context.arg.SkipWhitespace();
		context.arg.String(aFileNameOut);
		
		textDecoder := Codecs.GetTextDecoder(aOldEncoding);
		IF textDecoder = NIL THEN
			Message(context.out, 'No decoder found: "' + aOldEncoding + '"' + CRLF);
			RETURN
		END;
		textEncoder := Codecs.GetTextEncoder(aNewEncoding);
		IF textEncoder = NIL THEN
			Message(context.out, 'No encoder found: "' + aNewEncoding + '"' + CRLF);
			RETURN
		END;
		
		text := DecodeFile(context.out, aFileNameIn, textDecoder, res);
		IF res # Codecs.ResOk THEN
			Message(context.out, 'Decoding failed.' + CRLF);
			RETURN
		END;
		
		IF aFileNameOut # "" THEN
			EncodeFile(context.out, aFileNameOut, textEncoder, text, bBOM, res)
		ELSE
			EncodeFile(context.out, aFileNameIn, textEncoder, text, bBOM, res)
		END;
		IF res # Codecs.ResOk THEN
			Message(context.out, 'Encoding failed.' + CRLF);
			RETURN
		END
		
	END
END ChangeFileEncoding;

BEGIN
END OberonConsoleUtils.

System.Free OberonConsoleUtils ~
